var template = require('babel-template');

const componentTpl = template(`
    ko.components.register(NAME, {
        defaults: DEFAULTS,
        style: STYLE,
        template: TEMPLATE,
        createViewModel: function (params, componentInfo) {
            function CONSTRUCTOR (opts, info) {
                BODY
                mix(this, opts, MIXINS);
            }

            Object.assign(CONSTRUCTOR.prototype, METHODS);

            return new CONSTRUCTOR(params, componentInfo);
        }
    });
`);

module.exports = function (babel) {
    var t = babel.types;
    var styleAst = t.nullLiteral();
    var templateAst = t.nullLiteral();
    var mixinsAst = t.nullLiteral();
    var methodsAst = t.nullLiteral();
    var defaultsAst = t.nullLiteral();
    var constructorBodyAst = t.noop();

    var renderComponentAst = function () {
        return componentTpl({
            NAME: t.stringLiteral('comp'),
            CONSTRUCTOR: t.Identifier('Comp'),
            STYLE: styleAst,
            TEMPLATE: templateAst,
            MIXINS: mixinsAst,
            DEFAULTS: defaultsAst,
            METHODS: methodsAst,
            BODY: constructorBodyAst
        });
    };

    var variableVisitor = {
        VariableDeclarator: function (path) {
            switch (path.node.id.name) {
                case 'constructor':
                    constructorBodyAst = path.node.init.body.body;
                    break;

                case 'style':
                    styleAst = path.node.init;
                    break;

                case 'template':
                    templateAst = path.node.init;
                    break;

                case 'mixins':
                    mixinsAst = path.node.init;
                    break;

                case 'methods':
                    methodsAst = path.node.init;
                    break;

                case 'defaults':
                    defaultsAst = path.node.init;
                    break;
            }
        }
    };

    var pathList = [];

    return {
        visitor: {
            ExportNamedDeclaration: function (path) {
                path.traverse(variableVisitor);
                pathList.push(path);
            },
            Program: {
                exit: function () {
                    var lastPath = pathList[pathList.length - 1];

                    if (!t.isNoop(constructorBodyAst)) {
                        lastPath.insertAfter(renderComponentAst());
                        pathList.forEach(function (path) {
                            path.remove();
                        });
                    }
                }
            }
        }
    };
};
